package jsystem;

import java.util.*;
import kjava.io.*;
import java.io.*;
import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;
import javax.microedition.io.*;
import javax.microedition.io.file.*;



public class JConsole extends Form implements CommandListener,ItemStateListener{
  	private TextField commandField=new TextField("Input"," ",100,TextField.ANY );
  	private ScrollPane output	=new ScrollPane("Output",getWidth()-2,getHeight()/2+10);

	private Command enterCmd   = new Command("Enter", Command.OK, 1);
	private Command prvCmd     = new Command("<", Command.ITEM, 1);
	private Command nxtCmd     = new Command(">", Command.ITEM, 1);
	private Command upCmd      = new Command("^", Command.ITEM, 1);
	private Command dnCmd      = new Command("v", Command.ITEM, 1);
	private Command fileCmd    = new Command("File", Command.ITEM, 4);
	private Command winCmd     = new Command("Desktop", Command.ITEM, 4);
	private Command eofCmd     = new Command("EOF", Command.ITEM, 4);
  
  // Einige Sonderzeichen sind auf Mentasten gelegt
	private Command dach       = new Command("C ^", Command.ITEM, 2);
	private Command vert       = new Command("C |", Command.ITEM, 2);
  

  	private boolean sendEOF = false;

  	Vector History = new Vector();
  	int   HistoryIndex = 0;
	int   maxHistory = 100;
   

	private String Title = null;
	
	void setBusy(){
		if(Title==null)
			Title=getTitle();
		setTitle(Title+": ...working.");
	}
	
	void restoreTitle(){
		if(Title != null)
			setTitle(Title);
	}


	class Output extends java.io.OutputStream{
    	public void write(int b) throws IOException{
       		if(output==null){
           		throw new IOException("Shell not initialized");
       		}else
           		output.write((char)b);
    	}

    	public void write(byte[] b,
                  int off,
                  int len)
           throws IOException{
       		if(output==null){
           		throw new IOException("Shell not initialized");
       		}else
           		output.write(b,off,len);
    	}

    	public void write(byte[] b)
           	throws IOException{
           	write(b,0,b.length);
    	}
	}
    
    

  	class Input extends java.io.InputStream{
      	String text="";
      	int curChar=0;

      	public synchronized void append(String s){
          	text=s;
          	curChar=0;
          	notify();
      	}

      	public int available(){
         	return text.length()-curChar;
      	}

      	public synchronized int read() throws java.io.IOException{
         	try{
            	if (curChar==text.length()){
                	// Reset queue
                	curChar=0;
                	text = "";  
					restoreTitle();                
                	wait();
            	}

            	if(sendEOF){
                	sendEOF = false;
					setBusy();
                	return -1;
            	}
				setBusy();
            	return text.charAt(curChar++);
         	}catch (InterruptedException e){
           		throw new java.io.IOException();
         	}
     	}

      	public synchronized int read(byte[] b) throws IOException{
         	if(b==null)
             	throw new IOException();
         	try{
            	if (curChar==text.length()){
                	// Reset queue
                	curChar=0;
                	text = "";   
					restoreTitle();            
                	wait();
            	}

            	if(sendEOF){
                	sendEOF = false;
					setBusy();
                	return -1;
            	}
            	int anzahl = Math.min(b.length,text.length()-curChar);
            	System.arraycopy(text.getBytes(),curChar,b,0, anzahl);
            	if(anzahl<text.length()-curChar) 
                	curChar+=anzahl;
            	else{
                	text = "";
                	curChar=0;
            	}
				setBusy();
            	return anzahl;
         	}catch (InterruptedException e){
           		throw new java.io.IOException();
         	}
     	}
	 
		public synchronized int readChars(char[] c, int off, int len) 
			throws IOException{
			try{
				if (curChar==text.length()){
        		// Reset queue
					curChar=0;
     				text = "";
					restoreTitle();          
      				wait();
				}
				int	 anzahl = Math.min(len,text.length()-curChar);
				for(int i=0; i<anzahl; i++)
					c[off+i]=text.charAt(i+curChar);

				if(anzahl<text.length()-curChar) 
					curChar+=anzahl;
				else{
					text = "";
					curChar=0;
				}
				setBusy();
     			return anzahl;
			}catch(InterruptedException e){
          		throw new java.io.IOException();
      		}
		}
  	}
 
  // Reader which returns after pressing "Enter"
 
 	class ConsoleReader extends java.io.InputStreamReader{ 
		Input is;
		
  		public ConsoleReader(Input is){
			super(is);
			this.is = is;
		}
	
  		public int read(char[] cbuf,
                int off,
                int len)
        	throws IOException{
		 
		 	if(len>cbuf.length-off)
				len=cbuf.length-off;
			return is.readChars(cbuf, off, len);
		}
	}
  
  
	public Input         stdin  = new Input();
	public ConsoleReader conin  = new ConsoleReader( stdin );
	public PrintStream   stdout = new PrintStream(new Output());



	public void itemStateChanged(Item item){
		if(item==commandField){
			String t = commandField.getString();
			if(t.endsWith(JSystem.ls))
				processInput(t);
		}
	}
	
	void processInput(String t){
		if(t==null || t.length()==0)
			return;
		while(t.endsWith(JSystem.ls)){
			t = t.substring(0,t.indexOf(JSystem.ls));
		}
		History.addElement(t);
		if(History.size() > maxHistory)
			History.removeElementAt(0);
        stdout.println(">> " + t);
        commandField.setString(" ");
        HistoryIndex=History.size();
        t = t+ JSystem.ls;
        ((Input)stdin).append(t);
	}
			
		
						  	

	public void commandAction(Command c,
                          Displayable d){
     	if (d == this ){
        	if( c == enterCmd) {
				processInput(commandField.getString());
  			}else if(c==prvCmd){
     			if(HistoryIndex>0){
               		commandField.setString((String)History.elementAt(HistoryIndex-1));
               		HistoryIndex--;
    			}else
          			commandField.setString(" ");
         	}else if(c==nxtCmd){
                if(HistoryIndex<History.size()){
              		HistoryIndex++;
                    commandField.setString((String)History.elementAt(HistoryIndex-1));
                }else
                    commandField.setString(" ");
         	}else if(c==upCmd){
           		output.scrollUp();
         	}else if(c==dnCmd){
          		output.scrollDown();
         	}else if(c==fileCmd){
				new Thread(new Runnable() {
           			public void run() {
						if(JSystem.browser != null){
                  			JSystem.browser.updateDirlist();
							JSystem.display.setCurrent(JSystem.browser);
						}
                	}
            	}).start();
         	}else if(c==winCmd){
            	JSystem.showDesktop();
         	}else if(c==eofCmd){
            	sendEOF = true;
                ((Input)stdin).append(JSystem.ls);
         	}else if(c==dach){
				commandField.insert("^",commandField.getCaretPosition());
			}else if(c==vert){
				commandField.insert("|",commandField.getCaretPosition());
			}			
		}
   } 


	public JConsole() {
  		super("");

     	append(output);
     	append(commandField);
		

     	addCommand(enterCmd);
     	addCommand(prvCmd);
     	addCommand(nxtCmd);
     	addCommand(upCmd);
     	addCommand(dnCmd);
     	addCommand(fileCmd);
     	addCommand(winCmd);
     	addCommand(eofCmd);
     	addCommand(dach);
     	addCommand(vert);
     	setCommandListener(this);
		setItemStateListener(this);
     //setItemStateListener(this);    
  	}
}

